;**********************************************************************
;   This file is a basic code template for assembly code generation   *
;   on the PICmicro PIC16F876A. This file contains the basic code     *
;   building blocks to build upon.                                    *  
;                                                                     *
;   If interrupts are not used all code presented between the ORG     *
;   0x004 directive and the label main can be removed. In addition    *
;   the variable assignments for 'w_temp' and 'status_temp' can       *
;   be removed.                                                       *                         
;                                                                     *
;   Refer to the MPASM User's Guide for additional information on     *
;   features of the assembler (Document DS33014).                     *
;                                                                     *
;   Refer to the respective PICmicro data sheet for additional        *
;   information on the instruction set.                               *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Filename:	    s65_display.asm                                   *
;    Date:          11.2005                                           *
;    File Version:  0.1                                               *
;                                                                     *
;    Author:        Mario Konegger                                    *
;    Company:                                                         *
;                                                                     * 
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files required:                                                  *
;                                                                     *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes: This code implements a sample for steering an S65-Display *
;	    with an PIC microcontroller (Type LS020...). The PIC Fosz is  *
;		assumed to be 2MHz. In case of lower/faster speed, please	  *
;		adjust the wait-loop.			      						  *
;                                                                     *
;**********************************************************************


	list		p=16f876A	; list directive to define processor
	#include	<p16f876A.inc>	; processor specific variable definitions
	
	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_OFF & _PWRTE_ON & _XT_OSC & _WRT_OFF & _LVP_OFF & _CPD_OFF


;***** CONSTANTS DEFINITIONS ******************************************

#define		DISPLAY_CMD	PORTB,7	; Where the lines to the CS and CMD/DATA
#define		DISPLAY_RESET	PORTB,6	; Finally the Reset-line. (Output)
#define		DISPLAY_SELECT	PORTB,5	; are connected to the PIC. (output)
;#define	__DISPLAY_ROT		; Uncomment if you want output to be rotated.
#define		CHAR_WIDTH	D'8'
#define		CHAR_HEIGHT	D'14'
#define 	DISPLAY_WIDTH 	D'132'
#define 	DISPLAY_HEIGHT	D'176'
#define		TEXT_COL	D'16'
#define		TEXT_ROW	D'12'

#define		TEXTCOLOR		0xffff
#define		BACKCOLOR		0x0000
#define		CLRSCRCOLOR		0xffff


; Define the segment for the LCD-Display routines. These include the
; temporary values to store as well as transform the numbers to decimal
; format.
lcd udata_ovr	0x2d
RES0			res	1
RES1			res	1
NUM0			res	1
NUM1			res	1
TEMPDIV			res	1
LOOPCOUNTER0	res	1
LOOPCOUNTER1	res	1
SIGNDIGIT		res	1	; Here we place the sign as well
DIGIT32			res	1	; the digits in.
DIGIT10			res	1
X_POS			res	1
Y_POS			res	1
COUNTER			res	1


;**********************************************************************
	CODE	0x000		; processor reset vector

	nop					; nop required for icd
  	goto    	main	; go to beginning of program


;**********************************************************************
; SPI-transmission
spi_send
	movwf		SSPBUF
	goto		$+1
	goto		$+1
	goto		$+1
	return


spi_send_byte macro arg
	movlw		arg
	call		spi_send
	endm


spi_send_word macro arg
	movlw		high(arg)
	call		spi_send
	movlw		low(arg)
	call		spi_send
	endm

;**********************************************************************
; Initialize the S65 Display
init_display
	bsf			DISPLAY_RESET
	movlw		D'10'
	call		wait_ms
	bcf			DISPLAY_RESET
	bsf			DISPLAY_SELECT
	bsf			DISPLAY_CMD
	movlw		D'1'
	call		wait_ms
	bsf			DISPLAY_RESET
	movlw		D'10'
	call		wait_ms
	bsf			DISPLAY_SELECT
	movlw		D'1'
	call		wait_ms
	bsf			DISPLAY_CMD

	bcf			DISPLAY_SELECT
	spi_send_word	0xfdfd
	spi_send_word	0xfdfd

; wait, then init1 (50ms min. required acc. WEB, 68ms implemented)
;	movlw		D'68'
;	call		wait_ms

	spi_send_word	0xef00
	spi_send_word	0xee04
	spi_send_word	0x1b04
	spi_send_word	0xfefe
	spi_send_word	0xfefe
	spi_send_word	0xef90
	spi_send_word	0x4a04
	spi_send_word	0x7f3f
	spi_send_word	0xee04
	spi_send_word	0x4306
	bsf			DISPLAY_SELECT
	movlw		D'7'
	call		wait_ms		; wait for init2 (7ms, maybe 6ms sufficient, todo: test 6ms)
	bcf			DISPLAY_SELECT
	spi_send_word	0xef90
	spi_send_word	0x0983
	spi_send_word	0x0800
	spi_send_word	0x0baf
	spi_send_word	0x0a00
	spi_send_word	0x0500
	spi_send_word	0x0600

	spi_send_word	0x0700
	spi_send_word	0xef00
	spi_send_word	0xee0c
	spi_send_word	0xef90
	spi_send_word	0x0080
	spi_send_word	0xefb0
	spi_send_word	0x4902

	spi_send_word	0xef00
	spi_send_word	0x7f01
	spi_send_word	0xe181
	spi_send_word	0xe202
	spi_send_word	0xe276
	spi_send_word	0xe183
	spi_send_word	0x8001

; write inital screen command (maybe not for any type of display necessary).
	spi_send_word	0xef90
	spi_send_word	0x0000
	bsf			DISPLAY_SELECT
;	call		clrscr_display		; Can be skipped when clrscr is directly
;	return							; the following sub-routine.

clrscr_display
	bsf			DISPLAY_CMD
	bcf			DISPLAY_SELECT
	spi_send_word	0xef90
	spi_send_word	0x0500
	spi_send_word	0x0600
	spi_send_word	0x0700
	bcf			DISPLAY_CMD
	movlw		0x5b				; high-byte needs to be incremented by one.
	movwf		LOOPCOUNTER0
	movlw		0xc0
	movwf		LOOPCOUNTER1
clrscr
	spi_send_word	CLRSCRCOLOR
	decfsz		LOOPCOUNTER1,f
	goto		clrscr
	decfsz		LOOPCOUNTER0,f
	goto		clrscr
	bsf			DISPLAY_SELECT
	return


shutdown_display
	bsf			DISPLAY_CMD
	bcf			DISPLAY_SELECT
	spi_send_word	0xef00
	spi_send_word	0x1b04
	spi_send_word	0xfefe
	spi_send_word	0xfefe
	spi_send_word	0x7e04
	spi_send_word	0xe304
	spi_send_word	0xe404
	spi_send_word	0xe201
	spi_send_word	0x8000
	spi_send_word	0xe001
	spi_send_word	0x7f01
	movlw		D'5'
	call		wait_ms
	spi_send_word	0xe000
	spi_send_word	0x7f01
	movlw		D'5'
	call		wait_ms
	spi_send_word	0x0101
	bsf			DISPLAY_SELECT
	return


; This function outputs a number onto the display. A single letter
; is output. The following chars are supported:
; 0-9, -, =, p, T, H
; Input:
; X_POS .... the X-position of the beginning of the character
; Y_POS .... the Y-position of the beginning of the character
; w ........ the number of the character
; The following numbering applies: '0'-'9' => 0...9, '-' => 10
; Output:
; X_POS .... will be set for writing the next character
;
; Temporary values used: LOOPCOUNTER0, LOOPCOUNTER1, RES1
putchar_display
	movwf		RES1
	sublw		D'10'
	movlw		D'10'
	btfss		STATUS,C
	movwf		RES1
	incf		RES1,f
	movlw		ch_table - CHAR_HEIGHT
	addlw		CHAR_HEIGHT
	decfsz		RES1,f
	goto		$-2
#ifndef __DISPLAY_ROT
	addlw		CHAR_HEIGHT
	addlw		0xff
#endif
	movwf		RES1				; Calculate offset to EEPROM.
	bsf			DISPLAY_CMD
	bcf			DISPLAY_SELECT
	spi_send_word	0xef90
	spi_send_byte	0x05
#ifdef __DISPLAY_ROT
	spi_send_byte	0x00
	spi_send_byte	0x08
	movf		X_POS,w
	call		spi_send
	spi_send_byte	0x0a
	movf		Y_POS,w
	call		spi_send
	spi_send_byte	0x09
	movf		X_POS,w
	addlw		CHAR_WIDTH-1
	movwf		X_POS				; New-X-position now.
	call		spi_send
	spi_send_byte	0x0b
	movf		Y_POS,w
	addlw		CHAR_HEIGHT-1
	call		spi_send
#else
	spi_send_byte	0x04
	spi_send_byte	0x08
	movf		Y_POS,w
	call		spi_send
	spi_send_byte	0x0a
	movf		X_POS,w
	call		spi_send
	spi_send_byte	0x09
	movf		Y_POS,w
	addlw		CHAR_HEIGHT-1
	call		spi_send
	spi_send_byte	0x0b
	movf		X_POS,w
	addlw		CHAR_WIDTH-1
	movwf		X_POS				; New-X-position now.
	call		spi_send
#endif
	bsf			DISPLAY_SELECT
	bcf			DISPLAY_CMD			; From now on data follows.
	movlw		CHAR_HEIGHT
	movwf		LOOPCOUNTER0
	bcf			DISPLAY_SELECT
	bcf			INTCON,GIE			; Disable interrupts
write_char_outer
	movlw		CHAR_WIDTH
	movwf		LOOPCOUNTER1
	movf		RES1,w				; Load adress to be read-out from EEPROM.
#ifdef __DISPLAY_ROT
	incf		RES1,f				; and set to next byte.
#else
	decf		RES1,f
#endif
	errorlevel -302
	bsf			STATUS,RP1			; OK, lets prepare the EEPROM-Read
	movwf		EEADR
	bsf			STATUS,RP0
	bcf			EECON1,EEPGD
	bsf			EECON1,RD
	bcf			STATUS,RP0
	movf		EEDATA,w
	bcf			STATUS,RP1			; back to bank 0.
	errorlevel +302
	movwf		RES0				; Data-byte from EEPROM to RES0
write_char_inner
	rlf			RES0,f
	btfss		STATUS,C
	goto		no_pixel
;	spi_send_word		TEXTCOLOR
	movlw		high(TEXTCOLOR)
	movwf		SSPBUF
	goto		$+1
	goto		$+1
	goto		$+1
	goto		$+1
	movlw		low(TEXTCOLOR)
	movwf		SSPBUF
	goto		$+1
	goto		$+1
	goto		$+1
	goto		$+1
	goto		next_pixel
no_pixel
;	spi_send_word		BACKCOLOR
	movlw		high(BACKCOLOR)
	movwf		SSPBUF
	goto		$+1
	goto		$+1
	goto		$+1
	goto		$+1
	movlw		low(BACKCOLOR)
	movwf		SSPBUF
	goto		$+1
	goto		$+1
	goto		$+1
	goto		$+1
next_pixel
	decfsz		LOOPCOUNTER1,f
	goto		write_char_inner
	decfsz		LOOPCOUNTER0,f
	goto		write_char_outer
	bsf			DISPLAY_SELECT
	bsf			INTCON,GIE			; Enable interrupts again.
	return


; This function waits exactly the number of ms, as specified in 'w' register.
; Input: w ..... contains the ms to wait
; Notice: There return and call will consume four more clocks.
;		  This function needs to be adapted to the clock speed (2MHz assumed).
wait_ms
	movwf		LOOPCOUNTER1
	movlw		D'165'
	movwf		LOOPCOUNTER0
	movwf		LOOPCOUNTER0
	decfsz		LOOPCOUNTER0,f
	goto		$-1
	decfsz		LOOPCOUNTER1,f
	goto		$-6
	return



;**********************************************************************
; This is the main-program

main
	errorlevel	-302
	clrf		STATUS
	clrf		PORTB
	bsf			STATUS,RP0		; Initialize the SPI-interface
	bcf			TRISC,3			; for interfacting the display.
	bcf			TRISC,5
	bsf			TRISC,4			; Disable SDI (make corresponding PIN an output.
	clrf		TRISB			; All Port-B pins are output.
	movlw		B'10000000'
	movwf		SSPSTAT
	bcf			STATUS,RP0
	movlw		B'00100000'		; SSPEN at max. Fosz/4 speed.
	movwf		SSPCON

	call		init_display

	movlw		0x0a
	movwf		X_POS
	movwf		Y_POS
	movlw		D'10'
	movwf		COUNTER
next_digit
	decf		COUNTER,w
	call		putchar_display
	decfsz		COUNTER,f
	goto		next_digit

	goto		$			; Do nothing, but waiting :)
	errorlevel	+302

;**********************************************************************
; OK, here we have the table containing the bit-pattern of each char
; that will be available for output with putchar_display subroutine.

	org		0x2106

ch_table	de	0x00, 0x00, 0x00, 0x38, 0x6c, 0xc6, 0xc6, 0xd6, 0xc6, 0xc6, 0x6c, 0x38, 0x00, 0x00 ; '0' (48-32)
			de	0x00, 0x00, 0x00, 0x18, 0x38, 0x78, 0x18, 0x18, 0x18, 0x18, 0x18, 0x7e, 0x00, 0x00 ; '1'
			de	0x00, 0x00, 0x00, 0x7c, 0xc6, 0x06, 0x0c, 0x18, 0x30, 0x60, 0xc6, 0xfe, 0x00, 0x00 ; '2'
			de	0x00, 0x00, 0x00, 0x7c, 0xc6, 0x06, 0x06, 0x3c, 0x06, 0x06, 0xc6, 0x7c, 0x00, 0x00 ; '3'
			de	0x00, 0x00, 0x00, 0x0c, 0x1c, 0x3c, 0x6c, 0xcc, 0xfe, 0x0c, 0x0c, 0x1e, 0x00, 0x00 ; '4'
			de	0x00, 0x00, 0x00, 0xfe, 0xc0, 0xc0, 0xc0, 0xfc, 0x06, 0x06, 0xc6, 0x7c, 0x00, 0x00 ; '5'
			de	0x00, 0x00, 0x00, 0x38, 0x60, 0xc0, 0xc0, 0xfc, 0xc6, 0xc6, 0xc6, 0x7c, 0x00, 0x00 ; '6'
			de	0x00, 0x00, 0x00, 0xfe, 0xc6, 0x06, 0x0c, 0x18, 0x30, 0x30, 0x30, 0x30, 0x00, 0x00 ; '7'
			de	0x00, 0x00, 0x00, 0x7c, 0xc6, 0xc6, 0xc6, 0x7c, 0xc6, 0xc6, 0xc6, 0x7c, 0x00, 0x00 ; '8'
			de	0x00, 0x00, 0x00, 0x7c, 0xc6, 0xc6, 0xc6, 0x7e, 0x06, 0x06, 0x0c, 0x78, 0x00, 0x00 ; '9'
			de	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ; '-'


	END						; directive 'end of program'
